<?php

require('includes/application_top.php');

class N2GoApi
{

    private $apikey;
    private $username;
    private $connected = false;
    private $userId;

    /**
     * Asociative array with get parameters
     * @var array 
     */
    private $getParams;

    /**
     * Asociative array with post parameters
     * @var array 
     */
    private $postParams;

    public function __construct($action, $username, $apikey, $getParams = array(), $postParams = array())
    {
        if (tep_not_null($apikey) && tep_not_null($action)) {
            $this->apikey = $apikey;
            $this->username = $username;
            $this->getParams = $getParams;
            $this->postParams = $postParams;
            $this->connected = $this->checkApiKey();

            if (!$this->connected['success']) {
                echo $this->ping();
            } else {
                switch ($action) {
                    case 'getCustomers':
                        echo $this->getCustomers();
                        break;
                    case 'unsubscribeCustomer':
                        echo $this->unsubscribeCustomer();
                        break;
                    case 'getProduct':
                        echo $this->getProduct();
                        break;
                    case 'ping':
                        echo $this->ping();
                        break;
                    default:
                        break;
                }

                $this->logQuery($action);
            }
        } else {
            echo 'Error: Bad Request!';
        }
    }

    /**
     * Checks if user has connection
     * @return string
     */
    public function ping()
    {
        return json_encode($this->connected);
    }

    /**
     * 
     */
    public function getCustomers()
    {
        $hours = (isset($this->getParams['hours']) ? tep_db_prepare_input($this->getParams['hours']) : '');
        $subscribed = (isset($this->getParams['subscribed']) ? tep_db_prepare_input($this->getParams['subscribed']) : '');
        $limit = (isset($this->getParams['limit']) ? tep_db_prepare_input($this->getParams['limit']) : '');
        $offset = (isset($this->getParams['offset']) ? tep_db_prepare_input($this->getParams['offset']) : '');
        $queryWhere = '';
        $queryLimit = '';

        if (tep_not_null($limit)) {
            $offset = (tep_not_null($offset) ? $offset : 0);
            $queryLimit = ' LIMIT ' . $offset . ', ' . $limit;
        }

        if (tep_not_null($hours)) {
            $time = date('Y-m-d H:i:s', time() - 3600 * $hours);
            $queryWhere = " WHERE ci.customers_info_date_account_last_modified >= '$time'";
        }

        if (tep_not_null($subscribed) && $subscribed) {
            $queryWhere = (isset($time) ? ($queryWhere . ' AND ') : ' WHERE ') . 'cu.customers_newsletter=' . $subscribed;
        }

        $query = 'SELECT '
                . 'cu.customers_id AS crm_id, '
                . 'cu.customers_gender AS gender, '
                . 'cu.customers_firstname AS firstname, '
                . 'cu.customers_lastname AS lastname, '
                . 'date(cu.customers_dob) AS birthday, '
                . 'cu.customers_email_address AS mail, '
                . 'cu.customers_telephone AS mobile, '
                . 'cu.customers_newsletter AS subscribed, '
                . 'ab.entry_company AS company_name, '
                . 'ab.entry_street_address AS street, '
                . 'ab.entry_city AS city, '
                . 'co.countries_name '
                . 'FROM customers cu '
                . 'LEFT JOIN customers_info ci ON cu.customers_id = ci.customers_info_id '
                . 'LEFT JOIN address_book ab ON cu.customers_id = ab.customers_id '
                . 'LEFT JOIN countries co ON ab.entry_country_id = co.countries_id';

        $query .= $queryWhere . $queryLimit;

        $customersQuery = tep_db_query($query);
        $customers = array();
        while ($customer = tep_db_fetch_array($customersQuery)) {
            $customers[] = $customer;
        }

        $response = array(
            'success' => true,
            'message' => 'Success',
            'offset' => $offset,
            'limit' => $limit,
            'total' => $this->customersCount(),
            'customers' => $customers,
        );

        return json_encode($response);
    }

    public function unsubscribeCustomer()
    {
        $email = (isset($this->getParams['email']) ? tep_db_prepare_input($this->getParams['email']) : '');
        if (tep_not_null($email)) {
            tep_db_query("UPDATE customers SET customers_newsletter = 0 WHERE customers_email_address = '$email'");
            $result = tep_db_affected_rows();
            $response = ($result ? array('success' => true, 'message' => 'Customer successfully unsubscribed') :
                            array('success' => false, 'message' => 'There is no customer with given email'));
        } else {
            $response = array('success' => false, 'message' => 'Invalid parameter for email!');
        }

        return json_encode($response);
    }

    public function getProduct()
    {
        $id = isset($this->getParams['id']) ? tep_db_prepare_input($this->getParams['id']) : '';
        $lang = isset($this->getParams['lang']) ? tep_db_prepare_input($this->getParams['lang']) : '';

        if (!tep_not_null($id) || !tep_not_null($lang)) {
            return json_encode(array('success' => false, 'message' => 'Invalid or missing parameters for getProduct request!'));
        }

        $query = 'SELECT '
                . 'pr.products_id as id, '
                . 'pr.products_price as oldPrice, '
                . 'pr.products_price as newPrice, '
                . 'pr.products_price as oldPriceNet, '
                . 'pr.products_price as newPriceNet, '
                . 'pr.products_image as images, '
                . 'pd.products_name as name, '
                . 'pd.products_description as shortDescription, '
                . 'pd.products_description as description, '
                . 'pr.products_model as model, '
                . 'mf.manufacturers_name as brand, '
                . 'max(tr.tax_rate) as vat '
                . 'FROM products pr '
                . 'LEFT JOIN tax_rates tr ON pr.products_tax_class_id = tr.tax_class_id '
                . 'LEFT JOIN manufacturers mf ON pr.manufacturers_id = mf.manufacturers_id '
                . 'LEFT JOIN products_description pd ON pr.products_id = pd.products_id '
                . 'LEFT JOIN languages ln ON pd.language_id = ln.languages_id '
                . "WHERE pr.products_id = $id AND ln.code = '$lang'";

        $productsQuery = tep_db_query($query);
        $product = tep_db_fetch_array($productsQuery);
        if ($product) {
            if ($product['vat']) {
                $product['oldPrice'] = $product['newPrice'] = $product['oldPriceNet'] * (1 + $product['vat'] * 0.01);
                $product['vat'] = round($product['vat'] * 0.01, 2);
            }

            $product['oldPriceNet'] = $product['newPriceNet'] = round($product['oldPriceNet'], 2);
            $product['oldPrice'] = $product['newPrice'] = round($product['oldPrice'], 2);
            
            $product['url'] = HTTP_SERVER . DIR_WS_HTTP_CATALOG;
            $product['link'] = FILENAME_PRODUCT_INFO . '?products_id=' . $id;

            $product['images'] = ($product['images'] ? array(HTTP_SERVER . DIR_WS_HTTP_CATALOG . 'images/' . $product['images']) : array());
            $query = 'SELECT image FROM products_images WHERE products_id = ' . $id;
            $imagesQuery = tep_db_query($query);
            while ($image = tep_db_fetch_array($imagesQuery)) {
                $product['images'][] = HTTP_SERVER . DIR_WS_HTTP_CATALOG . 'images/' . $image['image'];
            }
        }

        $response = array(
            'success' => true,
            'message' => 'Success',
            'product' => $product,
        );

        return json_encode($response);
    }

    /**
     * Checks if there is an enabled user with given api key
     * @return array (
     *      'result'    =>   true|false,
     *      'message'   =>   result message,
     * )
     */
    private function checkApiKey()
    {
        $usersQuery = tep_db_query("SELECT * FROM newsletter2go_user WHERE apikey = '$this->apikey' AND email = '$this->username'");
        $user = tep_db_fetch_array($usersQuery);
        if (!empty($user)) {
            $this->userId = $user['id'];

            return $user['enabled'] ? array('success' => true, 'message' => 'pong') :
                    array('success' => false, 'message' => 'Your API key has been revoked! Contact your system administrator.');
        }

        return array('success' => false, 'message' => 'Invalid API key! Contact your system administrator.');
    }

    private function logQuery($info)
    {
        $info = tep_db_prepare_input($info);
        tep_db_query("INSERT INTO newsletter2go_log (user_id, info) VALUES($this->userId, '$info')");
    }

    private function customersCount()
    {
        $query = tep_db_query('SELECT COUNT(*) AS total FROM customers');
        $count = tep_db_fetch_array($query);

        return $count['total'];
    }

}

$user = isset($_SERVER['PHP_AUTH_USER']) ? $_SERVER['PHP_AUTH_USER'] : '';
$apikey = isset($_SERVER['PHP_AUTH_PW']) ? $_SERVER['PHP_AUTH_PW'] : '';
$action = isset($HTTP_GET_VARS['action']) ? $HTTP_GET_VARS['action'] : '';

$api = new N2GoApi($action, $user, $apikey, $HTTP_GET_VARS, $HTTP_POST_VARS);
